# Generated by Django 3.1.1 on 2020-11-08 00:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DiscordBot',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access this app'),),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Servers',
            fields=[
                ('server', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Server',
                'verbose_name_plural': 'Servers',
            },
        ),
        migrations.CreateModel(
            name='Channels',
            fields=[
                ('channel', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
                ('server', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='aadiscordbot.servers')),
            ],
            options={
                'verbose_name': 'Channel',
                'verbose_name_plural': 'Channels',
            },
        ),
    ]
