# Generated by Django 3.2.8 on 2022-01-04 08:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discord', '0003_big_overhaul'),
        ('aadiscordbot', '0002_reactionrolebinding_reactionrolemessage'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthBotConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('admin_users', models.ManyToManyField(to='discord.DiscordUser')),
            ],
        ),
    ]
